(* ============================================================================================= *)
(* FMOD Ex - Memory info header file. Copyright (c), Firelight Technologies Pty, Ltd. 2008-2011. *)
(*                                                                                               *)
(* Use this header if you are interested in getting detailed information on FMOD's memory        *)
(* usage. See the documentation for more details.                                                *)
(*                                                                                               *)
(* ============================================================================================= *)

(*
[STRUCTURE]
[
    [DESCRIPTION]
    Structure to be filled with detailed memory usage information of an FMOD object

    [REMARKS]
    Every public FMOD class has a getMemoryInfo function which can be used to get detailed information on what memory resources are associated with the object in question. 
    On return from getMemoryInfo, each member of this structure will hold the amount of memory used for its type in bytes.
    
    Members marked with [in] mean the user sets the value before passing it to the function.
    Members marked with [out] mean FMOD sets the value to be used after the function exits.


    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox, Xbox360, PlayStation 2, GameCube, PlayStation Portable, PlayStation 3

    [SEE_ALSO]
    System::getMemoryInfo
    EventSystem::getMemoryInfo
    FMOD_MEMBITS    
    FMOD_EVENT_MEMBITS
]
*)
type
  FMOD_MEMORY_USAGE_DETAILS = Record
    other: Cardinal;                          (* [out] Memory not accounted for by other types *)
    datastring: Cardinal;                     (* [out] String data *)
    system: Cardinal;                         (* [out] System object and various internals *)
    plugins: Cardinal;                        (* [out] Plugin objects and internals *)
    output: Cardinal;                         (* [out] Output module object and internals *)
    channel: Cardinal;                        (* [out] Channel related memory *)
    channelgroup: Cardinal;                   (* [out] ChannelGroup objects and internals *)
    codec: Cardinal;                          (* [out] Codecs allocated for streaming *)
    fileandstructures: Cardinal;              (* [out] File buffers and structures *)
    sound: Cardinal;                          (* [out] Sound objects and internals *)
    secondaryram: Cardinal;                   (* [out] Sound data stored in secondary RAM *)
    soundgroup: Cardinal;                     (* [out] SoundGroup objects and internals *)
    streambuffer: Cardinal;                   (* [out] Stream buffer memory *)
    dspconnection: Cardinal;                  (* [out] DSPConnection objects and internals *)
    dsp: Cardinal;                            (* [out] DSP implementation objects *)
    dspcodec: Cardinal;                       (* [out] Realtime file format decoding DSP objects *)
    profile: Cardinal;                        (* [out] Profiler memory footprint. *)
    recordbuffer: Cardinal;                   (* [out] Buffer used to store recorded data from microphone *)
    reverb: Cardinal;                         (* [out] Reverb implementation objects *)
    reverbchannelprops: Cardinal;             (* [out] Reverb channel properties structs *)
    geometry: Cardinal;                       (* [out] Geometry objects and internals *)
    syncpoint: Cardinal;                      (* [out] Sync point memory. *)
    eventsystem: Cardinal;                    (* [out] EventSystem and various internals *)
    musicsystem: Cardinal;                    (* [out] MusicSystem and various internals *)
    fev: Cardinal;                            (* [out] Definition of objects contained in all loaded projects e.g. events, groups, categories *)
    memoryfsb: Cardinal;                      (* [out] Data loaded with registerMemoryFSB *)
    eventproject: Cardinal;                   (* [out] EventProject objects and internals *)
    eventgroupi: Cardinal;                    (* [out] EventGroup objects and internals *)
    soundbankclass: Cardinal;                 (* [out] Objects used to manage wave banks *)
    soundbanklist: Cardinal;                  (* [out] Data used to manage lists of wave bank usage *)
    streaminstance: Cardinal;                 (* [out] Stream objects and internals *)
    sounddefclass: Cardinal;                  (* [out] Sound definition objects *)
    sounddefdefclass: Cardinal;               (* [out] Sound definition static data objects *)
    sounddefpool: Cardinal;                   (* [out] Sound definition pool data *)
    reverbdef: Cardinal;                      (* [out] Reverb definition objects *)
    eventreverb: Cardinal;                    (* [out] Reverb objects *)
    userproperty: Cardinal;                   (* [out] User property objects *)
    eventinstance: Cardinal;                  (* [out] Event instance base objects *)
    eventinstance_complex: Cardinal;          (* [out] Complex event instance objects *)
    eventinstance_simple: Cardinal;           (* [out] Simple event instance objects *)
    eventinstance_layer: Cardinal;            (* [out] Event layer instance objects *)
    eventinstance_sound: Cardinal;            (* [out] Event sound instance objects *)
    eventenvelope: Cardinal;                  (* [out] Event envelope objects *)
    eventenvelopedef: Cardinal;               (* [out] Event envelope definition objects *)
    eventparameter: Cardinal;                 (* [out] Event parameter objects *)
    eventcategory: Cardinal;                  (* [out] Event category objects *)
    eventenvelopepoint: Cardinal;             (* [out] Event envelope point objects *)
    eventinstancepool: Cardinal;              (* [out] Event instance pool memory *)
  end;


(*
[DEFINE]
[
    [NAME]
    FMOD_MEMBITS

    [DESCRIPTION]
    Bitfield used to request specific memory usage information from the getMemoryInfo function of every public FMOD Ex class.
    Use with the "memorybits" parameter of getMemoryInfo to get information on FMOD Ex memory usage.

    [REMARKS]
    Every public FMOD class has a getMemoryInfo function which can be used to get detailed information on what memory resources are associated with the object in question. 
    The FMOD_MEMBITS defines can be OR'd together to specify precisely what memory usage you'd like to get information on. See System::getMemoryInfo for an example.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, Solaris, iPhone

    [SEE_ALSO]
    FMOD_EVENT_MEMBITS
    System::getMemoryInfo
]
*)
const
  FMOD_MEMBITS_OTHER                       = $00000001;  (* Memory not accounted for by other types *)
  FMOD_MEMBITS_STRING                      = $00000002;  (* String data *)

  FMOD_MEMBITS_SYSTEM                      = $00000004;  (* System object and various internals *)
  FMOD_MEMBITS_PLUGINS                     = $00000008;  (* Plugin objects and internals *)
  FMOD_MEMBITS_OUTPUT                      = $00000010;  (* Output module object and internals *)
  FMOD_MEMBITS_CHANNEL                     = $00000020;  (* Channel related memory *)
  FMOD_MEMBITS_CHANNELGROUP                = $00000040;  (* ChannelGroup objects and internals *)
  FMOD_MEMBITS_CODEC                       = $00000080;  (* Codecs allocated for streaming *)
  FMOD_MEMBITS_FILE                        = $00000100;  (* Codecs allocated for streaming *)
  FMOD_MEMBITS_SOUND                       = $00000200;  (* Sound objects and internals *)
  FMOD_MEMBITS_SOUND_SECONDARYRAM          = $00000400;  (* Sound data stored in secondary RAM *)
  FMOD_MEMBITS_SOUNDGROUP                  = $00000800;  (* SoundGroup objects and internals *)
  FMOD_MEMBITS_STREAMBUFFER                = $00001000;  (* Stream buffer memory *)
  FMOD_MEMBITS_DSPCONNECTION               = $00002000;  (* DSPConnection objects and internals *)
  FMOD_MEMBITS_DSP                         = $00004000;  (* DSP implementation objects *)
  FMOD_MEMBITS_DSPCODEC                    = $00008000;  (* Realtime file format decoding DSP objects *)
  FMOD_MEMBITS_PROFILE                     = $00010000;  (* Profiler memory footprint. *)
  FMOD_MEMBITS_RECORDBUFFER                = $00020000;  (* Buffer used to store recorded data from microphone *)
  FMOD_MEMBITS_REVERB                      = $00040000;  (* Reverb implementation objects *)
  FMOD_MEMBITS_REVERBCHANNELPROPS          = $00080000;  (* Reverb channel properties structs *)
  FMOD_MEMBITS_GEOMETRY                    = $00100000;  (* Geometry objects and internals *)
  FMOD_MEMBITS_SYNCPOINT                   = $00200000;  (* Sync point memory. *)
  FMOD_MEMBITS_ALL                         = $ffffffff;  (* All memory used by FMOD Ex *)
(* [DEFINE_END] *)


(*
[DEFINE]
[
    [NAME]
    FMOD_EVENT_MEMBITS

    [DESCRIPTION]
    Bitfield used to request specific memory usage information from the getMemoryInfo function of every public FMOD Event System class.
    Use with the "event_memorybits" parameter of getMemoryInfo to get information on FMOD Event System memory usage.

    [REMARKS]
    Every public FMOD Event System class has a getMemoryInfo function which can be used to get detailed information on what memory resources are associated with the object in question. 
    The FMOD_EVENT_MEMBITS defines can be OR'd together to specify precisely what memory usage you'd like to get information on. See EventSystem::getMemoryInfo for an example.

    [PLATFORMS]
    Win32, Win64, Linux, Linux64, Macintosh, Xbox360, PlayStation 2, PlayStation Portable, PlayStation 3, Wii, Solaris, iPhone

    [SEE_ALSO]
    FMOD_MEMBITS
    System::getMemoryInfo
]
*)                                               
const
  FMOD_EVENT_MEMBITS_EVENTSYSTEM           = $00000001;  (* EventSystem and various internals *)
  FMOD_EVENT_MEMBITS_MUSICSYSTEM           = $00000002;  (* MusicSystem and various internals *)
  FMOD_EVENT_MEMBITS_FEV                   = $00000004;  (* Definition of objects contained in all loaded projects e.g. events, groups, categories *)
  FMOD_EVENT_MEMBITS_MEMORYFSB             = $00000008;  (* Data loaded with registerMemoryFSB *)
  FMOD_EVENT_MEMBITS_EVENTPROJECT          = $00000010;  (* EventProject objects and internals *)
  FMOD_EVENT_MEMBITS_EVENTGROUPI           = $00000020;  (* EventGroup objects and internals *)
  FMOD_EVENT_MEMBITS_SOUNDBANKCLASS        = $00000040;  (* Objects used to manage wave banks *)
  FMOD_EVENT_MEMBITS_SOUNDBANKLIST         = $00000080;  (* Data used to manage lists of wave bank usage *)
  FMOD_EVENT_MEMBITS_STREAMINSTANCE        = $00000100;  (* Stream objects and internals *)
  FMOD_EVENT_MEMBITS_SOUNDDEFCLASS         = $00000200;  (* Sound definition objects *)
  FMOD_EVENT_MEMBITS_SOUNDDEFDEFCLASS      = $00000400;  (* Sound definition static data objects *)
  FMOD_EVENT_MEMBITS_SOUNDDEFPOOL          = $00000800;  (* Sound definition pool data *)
  FMOD_EVENT_MEMBITS_REVERBDEF             = $00001000;  (* Reverb definition objects *)
  FMOD_EVENT_MEMBITS_EVENTREVERB           = $00002000;  (* Reverb objects *)
  FMOD_EVENT_MEMBITS_USERPROPERTY          = $00004000;  (* User property objects *)
  FMOD_EVENT_MEMBITS_EVENTINSTANCE         = $00008000;  (* Event instance base objects *)
  FMOD_EVENT_MEMBITS_EVENTINSTANCE_COMPLEX = $00010000;  (* Complex event instance objects *)
  FMOD_EVENT_MEMBITS_EVENTINSTANCE_SIMPLE  = $00020000;  (* Simple event instance objects *)
  FMOD_EVENT_MEMBITS_EVENTINSTANCE_LAYER   = $00040000;  (* Event layer instance objects *)
  FMOD_EVENT_MEMBITS_EVENTINSTANCE_SOUND   = $00080000;  (* Event sound instance objects *)
  FMOD_EVENT_MEMBITS_EVENTENVELOPE         = $00100000;  (* Event envelope objects *)
  FMOD_EVENT_MEMBITS_EVENTENVELOPEDEF      = $00200000;  (* Event envelope definition objects *)
  FMOD_EVENT_MEMBITS_EVENTPARAMETER        = $00400000;  (* Event parameter objects *)
  FMOD_EVENT_MEMBITS_EVENTCATEGORY         = $00800000;  (* Event category objects *)
  FMOD_EVENT_MEMBITS_EVENTENVELOPEPOINT    = $01000000;  (* Event envelope point object+s *)
  FMOD_EVENT_MEMBITS_EVENTINSTANCEPOOL     = $02000000;  (* Event instance pool data *)
  FMOD_EVENT_MEMBITS_ALL                   = $ffffffff;  (* All memory used by FMOD Event System *)

(* All event instance memory *)
  FMOD_EVENT_MEMBITS_EVENTINSTANCE_GROUP =
    (FMOD_EVENT_MEMBITS_EVENTINSTANCE or
     FMOD_EVENT_MEMBITS_EVENTINSTANCE_COMPLEX or
     FMOD_EVENT_MEMBITS_EVENTINSTANCE_SIMPLE or
     FMOD_EVENT_MEMBITS_EVENTINSTANCE_LAYER or
     FMOD_EVENT_MEMBITS_EVENTINSTANCE_SOUND);

(* All sound definition memory *)
  FMOD_EVENT_MEMBITS_SOUNDDEF_GROUP =
    (FMOD_EVENT_MEMBITS_SOUNDDEFCLASS or
     FMOD_EVENT_MEMBITS_SOUNDDEFDEFCLASS or
     FMOD_EVENT_MEMBITS_SOUNDDEFPOOL);
(* [DEFINE_END] *)

